import { createRequire } from 'module';
import { createProjectPlugin, createWorkspacePlugin, ProjectKind } from '@quilted/sewing-kit';

const require = createRequire(import.meta.url);
/**
 * Adds basic Babel hooks that other plugins can attach configuration to.
 */


function babelHooks() {
  return createProjectPlugin({
    name: 'SewingKit.Babel',

    build({
      hooks
    }) {
      hooks(({
        waterfall
      }) => ({
        babelPlugins: waterfall(),
        babelPresets: waterfall(),
        babelExtensions: waterfall(),
        babelTargets: waterfall()
      }));
    },

    develop({
      hooks
    }) {
      hooks(({
        waterfall
      }) => ({
        babelPlugins: waterfall(),
        babelPresets: waterfall(),
        babelExtensions: waterfall(),
        babelTargets: waterfall()
      }));
    },

    test({
      hooks
    }) {
      hooks(({
        waterfall
      }) => ({
        babelPlugins: waterfall(),
        babelPresets: waterfall(),
        babelExtensions: waterfall(),
        babelTargets: waterfall()
      }));
    }

  });
}
/**
 * Adds basic Babel hooks that other plugins can attach configuration to
 * for configuring Babel at the workspace level.
 */

function babelWorkspaceHooks() {
  return createWorkspacePlugin({
    name: 'SewingKit.Babel.Workspace',

    build({
      hooks
    }) {
      hooks(({
        waterfall
      }) => ({
        babelPlugins: waterfall(),
        babelPresets: waterfall(),
        babelExtensions: waterfall(),
        babelTargets: waterfall()
      }));
    },

    develop({
      hooks
    }) {
      hooks(({
        waterfall
      }) => ({
        babelPlugins: waterfall(),
        babelPresets: waterfall(),
        babelExtensions: waterfall(),
        babelTargets: waterfall()
      }));
    },

    test({
      hooks
    }) {
      hooks(({
        waterfall
      }) => ({
        babelPlugins: waterfall(),
        babelPresets: waterfall(),
        babelExtensions: waterfall(),
        babelTargets: waterfall()
      }));
    }

  });
}
function babelRollup() {
  return createProjectPlugin({
    name: 'SewingKit.Babel.Rollup',

    build({
      project,
      configure
    }) {
      configure(({
        extensions,
        babelPresets,
        babelPlugins,
        babelTargets,
        babelExtensions,
        rollupPlugins
      }) => {
        rollupPlugins?.(async plugins => {
          const helpers = project.kind === ProjectKind.Package ? 'runtime' : 'bundled';
          const [{
            babel
          }, targets, baseExtensions, babelPresetsOption, babelPluginsOption] = await Promise.all([import('@rollup/plugin-babel'), babelTargets.run([]), extensions.run(['.mjs', '.cjs', '.js']), babelPresets.run([]), babelPlugins.run(helpers === 'runtime' ? [require.resolve('@babel/plugin-transform-runtime')] : [])]);
          const finalExtensions = await babelExtensions.run(baseExtensions);
          return [...plugins, babel({
            envName: 'production',
            extensions: finalExtensions,
            exclude: 'node_modules/**',
            babelHelpers: helpers,
            configFile: false,
            babelrc: false,
            skipPreflightCheck: true,
            // @ts-expect-error Babel types have not been updated yet
            targets,
            presets: babelPresetsOption,
            plugins: babelPluginsOption
          })];
        });
      });
    },

    develop({
      project,
      configure
    }) {
      configure(({
        extensions,
        babelPresets,
        babelPlugins,
        babelTargets,
        babelExtensions,
        rollupPlugins
      }) => {
        rollupPlugins?.(async plugins => {
          const helpers = project.kind === ProjectKind.Package ? 'runtime' : 'bundled';
          const [{
            babel
          }, targets, baseExtensions, babelPresetsOption, babelPluginsOption] = await Promise.all([import('@rollup/plugin-babel'), babelTargets.run([]), extensions.run(['.mjs', '.cjs', '.js']), babelPresets.run([]), babelPlugins.run(helpers === 'runtime' ? [require.resolve('@babel/plugin-transform-runtime')] : [])]);
          const finalExtensions = await babelExtensions.run(baseExtensions);
          return [...plugins, babel({
            envName: 'production',
            extensions: finalExtensions,
            exclude: 'node_modules/**',
            babelHelpers: helpers,
            configFile: false,
            babelrc: false,
            skipPreflightCheck: true,
            // @ts-expect-error Babel types have not been updated yet
            targets,
            presets: babelPresetsOption,
            plugins: babelPluginsOption
          })];
        });
      });
    }

  });
}

export { babelHooks, babelRollup, babelWorkspaceHooks };
